/*
	Copyright 2013-2014, Marten de Vries

	Licensed under the Apache License, Version 2.0 (the "License");
	you may not use this file except in compliance with the License.
	You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

	Unless required by applicable law or agreed to in writing, software
	distributed under the License is distributed on an "AS IS" BASIS,
	WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
	See the License for the specific language governing permissions and
	limitations under the License.
*/

"use strict";

require("./compat.js");

var PouchDB = require("pouchdb");
//remove the _pouch_ prefix, it's not needed anyway.
PouchDB.prefix = "";
//remove idb; it's slower than websql and less tested
var i = PouchDB.preferredAdapters.indexOf("idb");
if (i !== -1) {
	PouchDB.preferredAdapters.splice(i, 1);
}

var GQL = require("pouchdb-gql");
var geopouch = require("geopouch");
var QuickSearch = require("pouchdb-quick-search");
var search = require("pouchdb-search");
var Delta = require("delta-pouch");
var Relational = require("relational-pouch");
var Erase = require("pouchdb-erase");
require("pouchdb-all-dbs")(PouchDB);

var Auth = require("pouchdb-auth");
var List = require("pouchdb-list");
var Replicator = require("pouchdb-replicator");
var Rewrite = require("pouchdb-rewrite");
var Security = require("pouchdb-security");
var Show = require("pouchdb-show");
var Update = require("pouchdb-update");
var Validation = require("pouchdb-validation");
var readyPromise = require("pouchdb-seamless-auth")(PouchDB);
var wrappers = require("pouchdb-wrappers");
require("pouchdb-vhost")(PouchDB);

//register 3rd party plugins (if necessary)
PouchDB.plugin(GQL);
PouchDB.plugin(geopouch);
PouchDB.plugin(QuickSearch);
PouchDB.prototype.quickSearch = PouchDB.prototype.search;
PouchDB.plugin({search: search});
PouchDB.plugin(Delta);
PouchDB.plugin(Relational);
PouchDB.plugin(Erase);

//register 'home made' plugins
PouchDB.plugin(Auth);
PouchDB.plugin(List);
PouchDB.plugin(Replicator);
PouchDB.plugin(Rewrite);
PouchDB.plugin(Security);
PouchDB.plugin(Show);
PouchDB.plugin(Update);
PouchDB.plugin(Validation);

var extend = require("extend");
var bridge = require("./bridge.js")(readyPromise);

//adds PouchDB.new
wrappers.installStaticWrapperMethods(PouchDB, {});

//expose relevant api
module.exports = global.PouchDB = PouchDB;
global.Security = Security;
extend(global, bridge);
